import os
import json
import subprocess
from prettytable import PrettyTable
# import libs.Groups as Groups
# import libs.Group as Group
# import libs.Shortcut as Shortcut

CONFIG_FILE = "~/.config/kebihelp.json"

class Config():
    def __init__(self):
        config_file = os.path.expanduser(CONFIG_FILE)
        if (not os.path.isfile(config_file)):
            self.config = self.default()
            self.save()
        with open(config_file) as f:
            self.config = json.load(f)

    def default(self):
        return {
            'Parameters': {
                'cmd_focused_window': "hyprctl activewindow",
                "importer": {
                      "match_line": ".*::(.+?)::(.+?)::(.+?)::.*",
                      "position_group": 1,
                      "position_keybinding": 2,
                      "position_label": 3
                },
                "keys": {
                    "tab_previous": "Backtab", 
                    "tab_next": "Tab",
                    "scroll_down": "Down",
                    "scroll_up": "Up",
                    "quit": "Esc"
                },
                "layout": {
                    'n_columns': 3,
                    'spacing': 3,
                    'background_color': '#282C34',
                    "opacity": 0.95,
                    "tabs": {
                        "background_color": "#30343D",
                        "background_color_current": "#4F5564",
                        "color": "#FFFFFF",
                        "font": "Lato",
                        "font_size": 8,
                        "padding": 0
                    },
                    'group_name': {
                        'background_color': '#FF0000',
                        'color': '#000000',
                        'bold': True,
                        'stretched': False,
                        "padding": "0px 5px 0px 2px",
                        "font": "Lato",
                        "font_size": 8
                    },
                    "shortcut": {
                        "label": {
                          "color": "#EEEEEE",
                          "bold": False,
                          "font": "Lato",
                          "font_size": 8
                        },
                        "value": {
                          "color": "#EEEEEE",
                          "bold": True,
                          "font": "Lato",
                          "font_size": 8
                        },
                        "spacing": 5
                    },
                }
            },
            'Rules': {
                'default': 'default'
            },
            'Keybindings': {}
        }
    def save(self):
        config_file = os.path.expanduser(CONFIG_FILE)
        with open(config_file, 'w') as f:
            json.dump(self.config, f, indent=2)

    def add_keybinding(self, tab, group, keybinding, description):
        print("Importing keybinding to tab {}: Group {} - {} - {}".format(tab, group, keybinding, description))
        if not tab in self.config['Keybindings']:
            self.config['Keybindings'][tab] = {
                '_hidden': False,
            }

        if not group in self.config['Keybindings'][tab]:
            self.config['Keybindings'][tab][group] = { '_hidden': False, 'keybindings': [] } 
        
        found = False
        for idx, k in enumerate(self.config['Keybindings'][tab][group]['keybindings']):
            if k['value'] == keybinding:
                found = True
                self.config['Keybindings'][tab][group]['keybindings'][idx]['label'] = description
        if not found: 
            self.config['Keybindings'][tab][group]['keybindings'].append({
                'value': keybinding,
                'label': description
        })

    
    def output_table(self, tab_selection):
        table = PrettyTable(["\033[33mTab\033[0m", "\033[33mGroup\033[0m", "\033[33mKey\033[0m", "\033[33mLabel\033[0m", "\033[33mHidden\033[0m"])
        for tab_name, tab_conf in self.config['Keybindings'].items():
            if tab_selection != None and tab_selection != tab_name:
                continue
            hidden = False
            if '_hidden' in tab_conf and tab_conf['_hidden']:
                hidden = True 
            for group_name, group_conf in self.config['Keybindings'][tab_name].items():
                if group_name == '_hidden':
                    continue
                if '_hidden' in group_conf and group_conf['_hidden']:
                    hidden = True
                for keybinding in group_conf['keybindings']:
                    if '_hidden' in keybinding and keybinding['_hidden']:
                        hidden = True
                    table.add_row([tab_name, group_name, keybinding['value'], keybinding['label'], hidden])
        print(table)




#!/usr/bin/env python3
from PyQt5.QtWidgets import QApplication
import sys
import argparse
from kebihelp.parsers import Parsers
from kebihelp.config import Config
from kebihelp.gui import MainWindow

VERSION="0.1.2"

def main():
    parser = argparse.ArgumentParser(description="Kebihelp - The universal keybinder helper")

    parser.add_argument("-v", "--version", dest="version", action="store_true", help="Show the current version")
    subparsers = parser.add_subparsers(dest="command", help="Main action to perform")


    parser_import = subparsers.add_parser('import', help='Import new keybindings')
    parser_import.add_argument('-t', dest="import_tab", help="Import to this tab")
    parser_import.add_argument('-s', dest="import_source", help="Source to import")

    parser_keys = subparsers.add_parser('list', help='List the existing keybindings')
    parser_keys.add_argument("-t", "--tab", dest="list_tab", help="Select the tab")


    parser_tabs = subparsers.add_parser('tabs', help='List the tabs')

    parser_show = subparsers.add_parser('show', help='Show the keybindings')
    parser_show.add_argument("-t", "--tab", dest="show_tab", help="Show only this tab")
    parser_show.add_argument("-a", "--auto", action="store_true", dest="show_auto", help="Show the right tab depending of focused window")

    args = parser.parse_args()

    if args.version:
        print("Kebihelp - v"+VERSION)
        exit(0)

    if not args.command:
        parser.print_usage()
        exit(0)

    if args.command == 'import':
        parsers = Parsers(args.import_tab, args.import_source)
        parsers.import_bindings()

    if args.command == 'list':
        config = Config()
        config.output_table(args.list_tab)

    if args.command == 'show':
        app = QApplication(sys.argv)
        window = MainWindow(args.show_tab, args.show_auto)
        window.show()
        app.exec()

if __name__ == '__main__':
    main()




from PyQt5.QtWidgets import QApplication, QMainWindow, QPushButton, QWidget, QFrame,  QLabel, QVBoxLayout, QHBoxLayout, QGridLayout, QScrollArea, QStyle, QLayout, QSizePolicy, QStackedWidget
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtGui import QFont, QKeySequence
from kebihelp.config import Config
import subprocess
from functools import partial
import sys

class MainWindow(QMainWindow):
    def __init__(self, selected_tab, autodetect):
        super().__init__()
        self.selected_tab = selected_tab
        self.autodetect = autodetect
        self.current_tab_name = None
        self.config = Config()
        
        self.keymap = self.load_keymap()
        self.keybindings = self.config.config['Keybindings']
        self.set_current_tab()

        self.layout = self.config.config['Parameters']['layout']
        # self.all_groups = self.config.get_all_groups()
        self.tab_buttons = []
        self.set_main_window()
        self.set_tabs()
        self.change_tab_by_name(self.current_tab_name)
    
    def load_keymap(self):
        default = {
            'tab_previous': 'Backtab',
            'tab_next': 'Tab',
            'scroll_down': 'Down',
            'scroll_up': 'Up',
            'quit': 'Esc',
        }
        if 'keys' in self.config.config['Parameters']:
            return self.config.config['Parameters']['keys']
        else:
            return default

    def set_current_tab(self):
        if self.selected_tab != None:
            if self.selected_tab not in self.keybindings:
                print("The tab {} does not exist".format(self.selected_tab))
                sys.exit(1)
            self.current_tab_name = self.selected_tab

        elif self.autodetect:
            result = subprocess.run(self.config.config['Parameters']['cmd_focused_window'], shell=True, stdout=subprocess.PIPE) 
            result = result.stdout.decode('utf-8').strip()
            for window_name, tab_name in self.config.config['Rules'].items():
                if window_name in result:
                    self.current_tab_name = tab_name
                    break
        if self.current_tab_name == None:
            tabs = self.keybindings.keys()
            if len(tabs) == 0:
                print("No tabs found")
                sys.exit(1)
            if '_default' in self.config.config['Rules'].keys() and self.config.config['Rules']['_default'] in tabs:
                self.current_tab_name = self.config.config['Rules']['_default']
            else:
                self.current_tab_name = list(tabs)[0]

        if '_hidden' in self.keybindings[self.current_tab_name] and self.keybindings[self.current_tab_name]['_hidden'] == True:
            print("The tab {} is hidden".format(self.current_tab_name))
            sys.exit(1)

    def set_main_window(self):
        self.setWindowTitle("Kebihelp")
        self.setFont(QFont('Lato'))
        self.setStyleSheet("background-color: {}".format(self.layout['background_color']))

        self.setWindowOpacity(self.layout['opacity'])

        self.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)
        self.main_widget = QWidget(self)

        self.window_layout = QVBoxLayout()
        self.tabs_layout = QHBoxLayout()
        self.stacked_widget = QStackedWidget()

        for tab_name, tab_config in self.keybindings.items():
            if '_hidden' in tab_config and tab_config['_hidden'] == True:
                continue
            # if self.current_tab_name != tab_name:
            #     continue
            widget = self.get_tab_shortcuts(tab_name)
            self.stacked_widget.addWidget(widget)

        self.window_layout.addLayout(self.tabs_layout)
        self.window_layout.addWidget(self.stacked_widget)
        # self.window_layout.setSizeConstraint(QVBoxLayout.SetMinimumSize)
        self.window_layout.addStretch()
        self.main_widget.setLayout(self.window_layout)
        # self.setCentralWidget(self.main_widget)
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidget(self.main_widget)
        self.scroll_area.setFocusPolicy(Qt.NoFocus)
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setStyleSheet("QScrollBar {width:  0px;}")
        self.setCentralWidget(self.scroll_area)

        width = 900
        if 'width' in self.layout:
            width = self.layout['width']
        height = 900
        if 'height' in self.layout:
            height = self.layout['height']
        self.setFixedSize(width, height)

    def set_tabs(self):
        index = 0
        for tab_name, tab_config in self.keybindings.items():
            if '_hidden' in tab_config and tab_config['_hidden'] == True:
                continue
            # if self.current_tab_name != tab_name:
            #     continue
            self.tab_button = QPushButton(tab_name)
            conf = self.layout['tabs']
            styles = {'normal': [], 'pressed':[]}
            styles['normal'].append("background-color: {}".format(conf['background_color']))
            styles['normal'].append("padding-top: {}".format(conf['padding']))
            styles['normal'].append("padding-bottom: {}".format(conf['padding']))
            styles['normal'].append("color: {}".format(conf['color']))
            styles['pressed'].append("background-color: {}".format(conf['background_color_current']))

            style = "QPushButton {" + '; '.join(styles['normal'])+ "} "
            style += "QPushButton:checked {" + '; '.join(styles['pressed'])+ "} "
            style += "QPushButton:hover { border: none}"
            style += "QPushButton:enabled { border: none}"

            self.tab_button.setFont(QFont(conf['font'], conf['font_size']))
            self.tab_button.setStyleSheet(style)
            self.tab_button.setCheckable(True)

            self.tab_button.clicked.connect(partial(lambda index: self.change_tab(index), index=index))
            self.tab_button.page = index
            self.tab_button.setAutoExclusive(True)
            self.tab_buttons.append(self.tab_button)
            # if index == 0:
            #     self.tab_buttons[index].setChecked(True)
            self.tabs_layout.addWidget(self.tab_button)
            index += 1

    def change_tab(self, index):
        self.stacked_widget.setCurrentIndex(index)
        self.tab_buttons[index].setChecked(True)

    def change_tab_by_name(self, tab_name):
        index = 0
        for tab in self.keybindings.keys():
            if tab == tab_name:
                self.change_tab(index)
                break
            if self.keybindings[tab]['_hidden'] == False:
                index += 1
            # if self.config.config['Tabs'][tab]['visible'] == True:
            #     index += 1
    

    def get_groups(self, tab):
        groups = {}
        for group_name, group_config in self.keybindings[tab].items():
            if group_name == '_hidden':
                continue
            if '_hidden' in group_config and group_config['_hidden'] == True:
                continue
            if len(group_config['keybindings']) == 0:
                continue
            groups[group_name] = { "lines": [], "n_lines": 0 } 
            
            for keybinding in group_config['keybindings']:
                if '_hidden' in keybinding and keybinding['_hidden'] == True:
                    continue
                shortcut_layout = self.get_shortcut_layout(keybinding)
                groups[group_name]['lines'].append(shortcut_layout)
                groups[group_name]['n_lines'] += 1

        groups_sorted = dict(sorted(groups.items(), key=lambda item: item[1]['n_lines']))
        return groups_sorted

    def get_tab_shortcuts(self, tab):
        
        groups = self.get_groups(tab)
        n_groups = len(groups.keys())

        widget = QWidget()

        # vertical_layout = QVBoxLayout()
        grid_layout = QGridLayout()
        
        current_row = 0
        current_column = 0
        for group_name, group in groups.items():
            block_layout = QVBoxLayout()
            block_layout.setSpacing(4)
            group_label = self.get_group_label(group_name)
            block_layout.addLayout(group_label)
            for line in group['lines']:
                block_layout.addLayout(line)
            block_layout.addStretch()
            grid_layout.addLayout(block_layout, current_row, current_column)
            current_column += 1
            
            if current_column >= 2:
                current_column = 0
                current_row += 1
        # columns_layout = QHBoxLayout()
        grid_layout.setVerticalSpacing(10) 
        grid_layout.setHorizontalSpacing(30) 
        grid_layout.setRowStretch(grid_layout.rowCount(), 1)
        widget.setLayout(grid_layout)
        return widget

    def get_group_label(self, group_name):
        hbox = QHBoxLayout()

        label = QLabel(group_name)
        font = QFont(self.layout['group_name']['font'], self.layout['group_name']['font_size'])
        if self.layout['group_name']['bold']:
            font.setBold(True)

        label.setFont(font)
        style = "background-color: {}; color: {}; padding: {}".format(self.layout['group_name']['background_color'], self.layout['group_name']['color'], self.layout['group_name']['padding'])
        label.setStyleSheet(style)
        hbox.addWidget(label)
        if not self.layout['group_name']['stretched']:
            hbox.addStretch()
        
        return hbox

    def get_shortcut_layout(self, shortcut):
        hbox = QHBoxLayout()
        # Label
        font_label = QFont(self.layout['shortcut']['label']['font'], self.layout['shortcut']['label']['font_size'])
        font_label.setBold(self.layout['shortcut']['label']['bold'])
        label = QLabel(shortcut['label'])
        style = "color: {};".format(self.layout['shortcut']['label']['color'])
        label.setStyleSheet(style)
        label.setFont(font_label)

        # Value
        font_value = QFont(self.layout['shortcut']['value']['font'], self.layout['shortcut']['value']['font_size'])
        font_value.setBold(self.layout['shortcut']['value']['bold'])
        value = QLabel(shortcut['value'])
        style = "color: {};".format(self.layout['shortcut']['value']['color'])
        value.setStyleSheet(style)
        value.setFont(font_value)

        hbox.addWidget(label)
        hbox.addStretch()
        hbox.addWidget(value)
        # hbox.setSpacing(self.layout['shortcut']['spacing'])
        # hbox.setContentsMargins(0, 0, 0, 0)
        return hbox

    def keyPressEvent(self, event):
        # if event.key() == Qt.Key_Escape:
        #     self.close()
        # if event.key() == Qt.Key_Q:
        #     self.close()
        if event.key() == QKeySequence(self.keymap['quit']):
            self.close()

        if event.key() == QKeySequence(self.keymap['tab_next']):
            current_index = self.stacked_widget.currentIndex()
            n_widgets = self.stacked_widget.count()
            if (current_index == n_widgets - 1):
                new_index = 0
            else:
                new_index = current_index + 1
            self.tab_buttons[new_index].setChecked(True)
            self.change_tab(new_index)

        if event.key() == QKeySequence(self.keymap['tab_previous']):
            current_index = self.stacked_widget.currentIndex()
            n_widgets = self.stacked_widget.count()
            if current_index == 0:
                new_index = n_widgets - 1
            else:
                new_index = current_index - 1
            self.tab_buttons[new_index].setChecked(True)
            self.change_tab(new_index)

        if event.key() == QKeySequence(self.keymap['scroll_down']):
            if self.scroll_area.verticalScrollBar().value() < self.scroll_area.verticalScrollBar().maximum():
                self.scroll_area.verticalScrollBar().setValue(self.scroll_area.verticalScrollBar().value() + self.scroll_area.verticalScrollBar().singleStep())

        if event.key() == QKeySequence(self.keymap['scroll_up']):
            if self.scroll_area.verticalScrollBar().value() > 0:
                self.scroll_area.verticalScrollBar().setValue(self.scroll_area.verticalScrollBar().value() - self.scroll_area.verticalScrollBar().singleStep())






import os
import re
import sys
import urllib.request
import glob
from kebihelp.config import Config
from prettytable import PrettyTable
import importlib

class Parsers():
    def __init__(self, destination_tab, source):
        self.destination_tab = destination_tab
        self.source = source
        self.config = Config()

    def import_from_quickref(self):
        req = urllib.request.Request(self.source, data=None, headers= {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36'})
        f = urllib.request.urlopen(req)
        html = f.read().decode('utf-8')
        sections = html.split('<div class="h3-wrap">')
        sections.pop(0)

        for section in sections:
            z = re.match(".*<h3.*</a>(.+?)</h3>", section, re.MULTILINE + re.DOTALL)
            title = z.group(1)
            trs = re.findall(r"<tr>(.+?)</tr>", section, re.MULTILINE + re.DOTALL)
            for tr in trs:
                tds = re.findall(r"<td>(.+?)</td>", tr ,re.MULTILINE + re.DOTALL)
                if (len(tds) > 1):
                    label = tds[1]
                    value = tds[0].replace("</code> <code>", "+").replace("</code>", "").replace('<code>', "")
                    self.config.add_keybinding(self.destination_tab, section, value, label)

    def import_from_directory(self):
        # Format: KBH=tab,value,description
        files = glob.glob(self.source+"/**/*", recursive=True)
        if not files:
            print("No files found")
            sys.exit(1)
        regexp = self.config.config['Parameters']['importer']['match_line']
        for file in files:
            # print("Importing from file "+file)
            if os.path.isfile(file):
                with open(file, 'r') as file:
                    lines = file.readlines()
                    for line in lines:
                        line = line.strip()
                        res = re.match(regexp, line, re.IGNORECASE)
                        if res:
                            group = res.group(self.config.config['Parameters']['importer']['position_group'])
                            keybind = res.group(self.config.config['Parameters']['importer']['position_keybinding'])
                            label = res.group(self.config.config['Parameters']['importer']['position_label'])
                            self.config.add_keybinding(self.destination_tab, group, keybind, label)


    def import_bindings(self):
        if 'quickref.me' in self.source:
            self.import_from_quickref()
        else:
            self.import_from_directory()
        self.config.save()


